#ifndef DETECTLINECUDA_H
#define DETECTLINECUDA_H

#include <opencv2/core.hpp>
#include <cufft.h>
#include <iostream>

#define DETECT_TILE_X 32
#define DETECT_TILE_Y 32
#define DETECT_UNROLL_Y 4

#define LOAD_FLOAT(i) d_Src[i]

#define ANGLE_0 0
#define ANGLE_23 1
#define ANGLE_45 2
#define ANGLE_68 3
#define ANGLE_90 4
#define ANGLE_113 5
#define ANGLE_135 6
#define ANGLE_157 7

class DetectLineCUDA
{
public:
    DetectLineCUDA(int cols, int rows, std::vector<int> _angles,int MinArea = 20,float Alpha = 1.15f,float Beta = 0.55f,float HairWidth = 12.0f);
	void detect(cv::cuda::GpuMat& src, cv::cuda::GpuMat& mask);
	~DetectLineCUDA();

private:
	int Width;
	int Height;
	int NumberOfFilter;
	int MinArea;
	int KernelRadius;
	int KernelW;
	int KernelH;
	int KernelX;
	int KernelY;
	int FFTW;
	int FFTH;
	float Alpha;
	float Beta;
	float HairWidth;

	float SigmaX;
	float SigmaY;

    std::vector<int> angles;

	float* _initGaborFilterCube();

	cufftHandle _fftPlanFwd;
	cufftHandle _fftPlanInv;

	int snapTransformSize(int dataSize);
	int iAlignUp(int a, int b);
	void _padKernel(float* d_Dst, float* d_Src);

	int iDivUp(int a, int b);
	void _modulateAndNormalize(cufftComplex* d_Dst, cufftComplex* d_DataSrc, cufftComplex* d_KernelSrc, int padding);
	void _cubeReduction(float* d_Src, uchar* d_Dst);
	void _padDataClampToBorder(float* d_Dst, float* d_Src);
};

#endif // DETECTLINECUDA_H
